var fs = require('fs');
var exec = require('child_process').exec;
var execSync = require('child_process').execSync;
var replaceStream = require('replacestream');
var copydir = require('copy-dir');

function fileExists(filePath) {
    try {
        return fs.statSync(filePath).isFile();
    } catch (err) {
        return false;
    }
}

exports.usage = '初始化项目';

exports.setOptions = function (optimist) {
};

exports.run = function (options) {

    var cwd = options.cwd;

    var packageJsonPath = sysPath.join(cwd, './package.json');
    var templatePath = options.hy2 ? sysPath.resolve(__dirname, '../template/hy') : sysPath.resolve(__dirname, '../template/default');

    var projectName = '';
    if (fileExists(packageJsonPath)) {
        try {
            projectName = JSON.parse(fs.readFileSync(packageJsonPath)).name;
        } catch (e) { }
    } else {
        projectName = sysPath.basename(cwd);
    }

    createPackageJson().then(function () {
        createConfigFile().then(function () {
            copyTemplate();
            installDependencies();
            buildLib();
        });
    });

    function createPackageJson() {
        return new Promise(function (resolve, reject) {
            try {
                var stream;
                stream = fs.createReadStream(sysPath.resolve(templatePath, 'package.json'))
                    .pipe(replaceStream('#_name', projectName))
                    .pipe(fs.createWriteStream(sysPath.resolve(cwd, 'package.json')));

                stream.on('finish', function () {
                    log('Successfully created ' + 'package.json' + ' file in ' + cwd);
                    resolve();
                });
            } catch (e) {
                error('Something wrong when created package.json: ' + e);
                reject();
            }
        });
    }

    function createConfigFile() {
        return new Promise(function (resolve, reject) {
            var configFileName = 'ykit.js';
            var stream;

            try {
                stream = fs.createReadStream(sysPath.resolve(templatePath, configFileName))
                    .pipe(replaceStream('#_name', projectName))
                    .pipe(fs.createWriteStream(sysPath.resolve(cwd, configFileName)));

                stream.on('finish', function () {
                    log('Successfully created ' + configFileName + ' file in ' + cwd);

                    if (fileExists(sysPath.resolve(cwd, 'ykit.yo.js'))) {
                        fs.unlinkSync(sysPath.resolve(cwd, 'ykit.yo.js'))
                    }

                    resolve();
                });
            } catch (e) {
                error('Something wrong when created ' + configFileName + ': ' + e);
                reject();
            }
        });
    }

    function copyTemplate() {
        try {
            copydir.sync(sysPath.resolve(templatePath, 'src'), sysPath.resolve(cwd, 'src'));
            fs.writeFileSync(sysPath.resolve(cwd, '.gitignore'), fs.readFileSync(sysPath.resolve(templatePath, 'gitignore.config'), 'utf-8'), 'utf-8');

            var _html = fs.readFileSync(sysPath.resolve(cwd, 'src/html/index.html'), 'utf-8').replace(/#_name/g, projectName);
            fs.writeFileSync(sysPath.resolve(cwd, 'src/html/index.html'), _html, 'utf-8');

            log('Successfully created src in ' + cwd);
        } catch (e) {
            error('Something wrong when created src: ' + e);
        }
    }

    function installDependencies() {
        var dependencies;
        var _cmd = options.hy2 ? 'npm i --save --registry http://registry.npm.corp.qunar.com/ ' : 'npm i --save --registry=https://registry.npm.taobao.org ';

        if (options.hy2) {
            dependencies = ['react@15.4.2', 'react-dom@15.4.2', 'babel-polyfill', 'yo3', 'yo-router', '@qnpm/hysdk'];
        } else {
            dependencies = ['react@15.4.2', 'react-dom@15.4.2', 'babel-polyfill', 'yo3', 'yo-router'];
        }

        for (var i = 0, len = dependencies.length; i < len; i++) {
            log('Installing ' + dependencies[i] + '...');
            execSync(_cmd + dependencies[i]);
        }

        log('Successfully install dependencies.');
    }

    function buildLib() {
        execSync('ykit dll');
        log('Successfully build lib.js.');
    }
};
