var fs = require('fs');
var sysPath = require('path');
var spawn = require('child_process').spawn;
var exec = require('child_process').exec;
var colors = require('colors');


var GIT_PERFIX = 'git@gitlab.corp.qunar.com:source/';

function log(data) {
    console.log('> 【执行日志】: ' + data.toString());
}

function execute_log(cmd, callback) {
    cmd.stdout.on('data', log);
    cmd.stderr.on('data', log);
    cmd.on('close', function(code) {
        if (code == 0) {
            if (callback) {
                callback();
            } else {
                console.log('√ 命令执行成功！'.green);
            }
        } else {
            console.log('X 命令执行失败！'.red);
        }
    });
}

function init(dir) {
    console.log('> 正在初始化 Source 工程 ......');
    execute_log(spawn('git', ['submodule', 'init'], {
        cwd: dir
    }), function() {
        execute_log(spawn('git', ['submodule', 'update'], {
            cwd: dir
        }), function() {
            execute_log(spawn('git', ['submodule', 'foreach', 'git', 'checkout', 'master'], {
                cwd: dir
            }), function() {
                execute_log(spawn('git', ['submodule', 'foreach', 'git', 'pull'], {
                    cwd: dir
                }));
            });
        });
    });
}

function update(dir) {
    console.log('> 正在更新 Source 工程 ......');
    execute_log(spawn('git', ['submodule', ' foreach', 'git', 'checkout', 'master'], {
        cwd: dir
    }), function() {
        execute_log(spawn('git', ['submodule', 'foreach', 'git', 'pull'], {
            cwd: dir
        }));
    });
}

function add(dir, name) {
    console.log(('> 开始添加 Source 工程: ' + name + ' ......').green);
    console.log(('> 工程地址为: ' + GIT_PERFIX + name + '.git').green);
    execute_log(spawn('git', ['submodule', 'add', GIT_PERFIX + name + '.git', 'src/source/' + name], {
        cwd: dir
    }));
}

function sync(dir, source) {
    console.log('> 正在同步 Source 工程 ......'.green);
    console.log('> 正在读取 .dev 文件 ......'.green);
    var host = null;
    try {
        host = JSON.parse(fs.readFileSync(sysPath.join(dir, '.dev')), 'utf-8').dev.host;
        if (!host) {
            throw new Error();
        }
    } catch (e) {
        console.log('X 读取 .dev 文件失败！'.red);
        return;
    }
    exec("rsync -rzcv --chmod=a='rX,u+w' --rsync-path='sudo rsync' ./ " + host + ':/home/q/www/' + source + '/ --exclude=.svn --exclude=.git --temp-dir=/tmp', {
        cwd: sysPath.join(dir, 'src/source/')
    }, function(err, stdout, stderr) {
        if (err) {
            console.log("[提示] 如遇问题参见 http://wiki.corp.qunar.com/display/fe/8+Trouble+shooting");
            throw err;
        }
        if (stdout) {
            console.log(stdout);
        }
        if (stderr) {
            console.error(stderr);
        }
    });
};

function publish(dir, message) {
    console.log('> 正在提交发布 Source 工程 ......');
    execute_log(spawn('git', ['submodule', 'foreach', 'git', 'add', '--all'], {
        cwd: dir
    }), function() {
        execute_log(spawn('git', ['submodule', 'foreach', 'git', 'commit', '-am', message], {
            cwd: dir
        }), function() {
            execute_log(spawn('git', ['submodule', 'foreach', 'git', 'push'], {
                cwd: dir
            }));
        });
    });
};


exports.usage = 'Source 工具';

exports.setOptions = function(optimist) {
    optimist.alias('i', 'init');
    optimist.describe('i', '初始化 Source 工程');
    optimist.alias('u', 'update');
    optimist.describe('u', '更新 Source 工程');
    optimist.alias('a', 'add');
    optimist.describe('a', '添加 Source 工程 ');
    optimist.alias('s', 'sync');
    optimist.describe('s', '同步到开发机');
    optimist.alias('p', 'publish');
    optimist.describe('p', '发布 Source');
};

exports.run = function(options) {

    options.publish = typeof options.p == "undefined" ? options.publish : options.p;
    options.init = typeof options.i == "undefined" ? options.init : options.i;
    options.update = typeof options.u == "undefined" ? options.update : options.u;
    options.add = typeof options.a == "undefined" ? options.add : options.a;
    options.sync = typeof options.s == "undefined" ? options.sync : options.s;

    if (options.init) {
        init(options.cwd);
        return;
    }

    if (options.add) {
        if (options.add == true) {
            console.log('∆ 请输入 Source 项目名。例如: fekit source --add hotel_qta');
        } else {
            add(options.cwd, options.add);
        }
        return;
    }

    if (options.update) {
        update(options.cwd);
        return;
    }

    if (options.publish) {
        if (options.publish == true) {
            console.log('∆ 请输入 Source 项目提交信息。例如: fekit source --publish 增加活动图片');
        } else {
            publish(options.cwd, options.publish);
        }
        return;
    }

    if (options.sync) {
        sync(options.cwd, 's.qunarzz.com');
        return;
    }

    console.log('X 命令不存在。使用 fekit source --help 查看相关命令。'.red);
};
