var fs = require('fs');
var exec = require('child_process').exec;
var _ = require('underscore');

exports.usage = '同步/上传当前目录至远程服务器(依赖rsync)';

exports.setOptions = function(optimist) {
    optimist.alias('i', 'include');
    optimist.describe('i', '同 rsync 的 include 选项');
    optimist.alias('e', 'exclude');
    optimist.describe('e', '同 rsync 的 exclude 选项');
    optimist.alias('d', 'delete');
    optimist.describe('d', '删除服务器上本地不存在的文件');

    // fekit之前的参数
    // optimist.alias('f', 'file');
    // optimist.describe('f', '更换其它的配置文件, 默认使用当前目录下的 .dev');
    // optimist.alias('n', 'name');
    // optimist.describe('n', '更换其它的配置名, 默认使用 dev');
    // optimist.alias('i', 'init');
    // optimist.describe('i', '初始化 .dev 文件');
    // optimist.alias('x', 'nonexec');
    // optimist.describe('x', '上传后禁止执行 shell');
};

exports.run = function(options) {
    var self = this;

    // 获取全局用户名
    var globalConfig = JSON.parse(fs.readFileSync(YKIT_RC, {encoding: 'utf8'}));

    if(globalConfig.user) {
        runSync(globalConfig.user)
    } else {
        process.stdout.write('首次sync, 请输入用户id: ');

        process.stdin.resume();
        process.stdin.setEncoding('utf8');

        process.stdin.on('data', function (text) {
            if (text && text !== '\n') {
                done(text);
            } else {
                warn('id不能为空');
                process.stdout.write('首次sync, 请输入用户id: ');
            }
        });
    }

    function done(inputText) {
        globalConfig.user = inputText.trim();
        fs.writeFile(YKIT_RC, JSON.stringify(globalConfig, null, '  '), {encoding: 'utf8'});

        runSync(globalConfig.user)
    }

    function runSync(user) {
        var conf, default_exclude, default_include, item, opts, yaml;

    	conf = self.project.config.getConfig().sync

    	if (options.name && yaml[options.name]) {
    		conf = yaml[options.name];
    	}
    	if (!conf) {
    		console.error("没有匹配的 " + (options.name || 'dev') + " 节点");
    		return;
    	}
    	opts = {
    		host: conf.host,
    		path: conf.path,
    		local: conf.local || './',
    		user: user,
    		shell: conf.shell
    	};

    	if (options["delete"]) {
    		opts["delete"] = true;
    	}
    	if (options.nonexec) {
    		opts.nonexec = true;
    	}

    	default_include = [];
    	if (conf['include'] && conf['include']['length'] > 0) {
    		default_include = default_include.concat(conf['include']);
    	}
    	if (options.include && options.include.length) {
    		default_include = default_include.concat(options.include);
    	}
    	if (default_include.length > 0) {
    		default_include = _.uniq(default_include);
    		opts.include = ((function() {
    			var i, len, results;
    			results = [];
    			for (i = 0, len = default_include.length; i < len; i++) {
    				item = default_include[i];
    				results.push("--include=" + item);
    			}
    			return results;
    		})()).join(' ');
    	}
    	default_exclude = ['.idea', '.svn', '.git', '.DS_Store', 'node_modules', 'prd', 'loc', 'env', 'dll'];
    	if (conf['exclude'] && conf['exclude'].length > 0) {
    		default_exclude = default_exclude.concat(conf['exclude']);
    	}
    	if (options.exclude && options.exclude.length) {
    		default_exclude = default_exclude.concat(options.exclude);
    	}
    	if (default_exclude.length > 0) {
    		default_exclude = _.uniq(default_exclude);
    		opts.exclude = ((function() {
    			var i, len, results;
    			results = [];
    			for (i = 0, len = default_exclude.length; i < len; i++) {
    				item = default_exclude[i];
    				results.push("--exclude=" + item);
    			}
    			return results;
    		})()).join(' ');
    	}
    	return rsync(opts);
    }
};

rsync = function(opts) {
	var _args, args;
	_args = ["-rzcv", "--chmod=a='rX,u+w'", "--rsync-path='sudo rsync'", "" + opts.local, "" + opts.user + "@" + opts.host + ":" + opts.path, "" + (opts.include || ''), "" + (opts.exclude || ''), "--temp-dir=/tmp"];
	if (opts["delete"]) {
		_args.push('--delete');
	}
	args = _args.join(' ');

    // TODO
	// if (utils.sys.isWindows) {
	// 	process.env['HOME'] = process.env.USERPROFILE;
	// }

	log("[调用] rsync " + args);
	return exec("rsync " + args, {
		maxBuffer: 200 * 1024 * 1024
	}, function(err, stdout, stderr) {
		// var common_shell;
		if (err) {
			log("[提示] 如遇问题参见 http://wiki.corp.qunar.com/display/fe/8+Trouble+shooting");
			throw err;
		}
		if (stdout) {
			log(stdout);
		}
		if (stderr) {
			console.error(stderr);
		}

        process.exit(1);

        // fekit脚本 暂时不执行
		// if (!opts.nonexec) {
		// 	common_shell = {
		// 		user: opts.user,
		// 		host: opts.host,
		// 		shell: "sudo /home/q/tools/bin/fekit_common_shell.sh \"" + opts.path + "\""
		// 	};
		// 	return shell(common_shell, function() {
		// 		if (opts.shell) {
		// 			return shell(opts);
		// 		}
		// 	});
		// }
	});
};

shell = function(opts, cb) {
	var args, cmd;
	cmd = opts.shell.replace(/'/g, "\\'");
	args = "" + opts.user + opts.host + " '" + cmd + "'";
	log("[执行] ssh " + args);
	return exec("ssh " + args, function(err, stdout, stderr) {
		if (err) {
			throw err;
		}
		if (stdout) {
			log(stdout);
		}
		if (stderr) {
			console.error(stderr);
		}
		return cb();
	});
};
