/**
 * 从js入口中提取所有的scss/css依赖并去重合并
 * @author jiao.shen
 */
var fs = require('fs');
var Path = require('path');
var normalize = require('normalize-path');
var _ = require('lodash');
var rimport = /@?import[^'",]*(?:(['"])([^'",]+)\1(.*)|url\((['"]?)([^)'"]+)\4\))/;
var rimportWithComma = /,\s*(['"])([^'",]+)\1\s*/;
var rrequire = /require(?:\.async)?\s*\((['"])([^'")]+)\1\)/;
var rscsspackage = /yo3|@qnpm\/yo/;
var rpossibleScssFileName = /([^\/]+\.scss)$/;
var rsinglelineComment = /\/\/.*(?:[\n\r]|$)/g;
var rmultilineComment = /\/\s*\*(?:(?!(\*\s*\/)).|[\n\r])*(?:\*\s*\/)/g;
var ryoComponentImport = /^\s*import\s+\{([^}]+)}\s+from\s+(['"])\$yo-component\2;?/;
var ryoComponentImportG = /^\s*import\s+\{([^}]+)}\s+from\s+(['"])\$yo-component\2;?/gm;

function combine(entrance, resolve, context, ignore) {

    function existsSync(filepath) {
        return filepath && fs.existsSync(filepath);
    }

    function isIgnored(abspath, rscsspackage) {
        return abspath && abspath.match(/node_modules/) && !abspath.match(rscsspackage);
    }

    function getScssDependencies(entrance, resolve, context, scsspackage) {
        if (!scsspackage) {
            scsspackage = rscsspackage;
        }
        var entranceExist = existsSync(entrance);
        if (entranceExist) {
            var content = fs.readFileSync(entrance, 'utf8')
                .replace(rsinglelineComment, '')
                .replace(rmultilineComment, '');
            if (content.match(ryoComponentImportG)) {
                content = content.replace(ryoComponentImportG, function ($0) {
                    return $0.replace(/[\n\r]/gm, '')
                });
            }
            var allLines = content.split(/[\n\r;]+/);
            var scssDeps = [];

            allLines.forEach(function (line) {
                function renderImport(importPath) {
                    var absImportPath = getAbsImportPath(entrance, importPath, resolve, context);
                    if (!isIgnored(absImportPath, scsspackage)) {
                        // 如果是scss, 直接push
                        if (isScssFile(absImportPath)) {
                            scssDeps = scssDeps.concat(getScssDependencies(absImportPath, resolve, context, scsspackage));
                            scssDeps.push(absImportPath);
                        } else { // 否则递归查找js依赖中的scss依赖
                            scssDeps = scssDeps.concat(getScssDependencies(absImportPath, resolve, context, scsspackage));
                        }
                    }
                }

                // 处理yo3的语法糖
                var mshortcut = line.match(ryoComponentImport);
                if (mshortcut) {
                    mshortcut[1]
                        .split(',')
                        .filter(function (name) {
                            return !!name;
                        })
                        .map(function (name) {
                            return name.trim().toLowerCase();
                        })
                        .forEach(function (name) {
                            renderImport('yo3/component/' + name);
                        });
                }

                var mimport = line.match(rimport);
                if (!mimport) {
                    mimport = line.match(rrequire);
                }

                if (mimport) {
                    var importPath = mimport[2] || mimport[5];
                    renderImport(importPath);

                    if (mimport[2] && mimport[3]) {
                        var rimportWithCommaG = new RegExp(rimportWithComma, 'g');
                        var tailImports = mimport[3].match(rimportWithCommaG);
                        if (tailImports) {
                            tailImports.forEach(function (imp) {
                                var mimp = imp.match(rimportWithComma);
                                var impPath = mimp[2];
                                if (mimp && impPath) {
                                    renderImport(impPath);
                                }
                            });
                        }
                    }
                }
            });

            return scssDeps;
        }
        return [];
    }

    function getAbsImportPath(entrance, importPath, resolve, context) {
        if (importPath.search('/') === -1) {
            importPath = importPath.replace(/\./g, '/').replace(/\/css/, '.css').replace(/\/scss/, '.scss');
        }
        if (importPath.charAt(0) === '~') {
            importPath = Path.resolve(context, 'node_modules', importPath.slice(1));
        }
        var ret = importPath;
        // 替换alias, 只替换业务代码中的别名
        var alias = resolve.alias || {};
        var aliasMatched = importPath.split('/').filter(function (dirname) {
            return Object.keys(alias).indexOf(dirname) !== -1;
        });
        var aliasFound = aliasMatched ? aliasMatched[0] : null;
        var aliasContent = alias[aliasFound];
        var isBizFolder = entrance.search('node_modules') === -1;

        if (aliasFound && isBizFolder) {
            importPath = importPath.replace(aliasFound, aliasContent);
        }
        // 优先尝试node_modules
        var nodeModule = tryLoadNodeModules(context, importPath);
        if (nodeModule && !entrance.match(/\.(css|scss)$/)) {
            ret = nodeModule;
        } else { // 最后尝试普通module
            ret = Path.resolve(Path.dirname(entrance), importPath);
            // 如果ret不存在，看看是不是fekit_modules里面的css
            if (Path.extname(entrance).match(/css/)) {
                if (!Path.isAbsolute(importPath) && !existsSync(ret)) {
                    var fekitPath = Path.resolve(context, 'fekit_modules');
                    if (existsSync(fekitPath)) {
                        var alternative = Path.resolve(fekitPath, importPath, 'src', 'index.css');
                        if (existsSync(alternative)) {
                            ret = alternative;
                        }
                    }
                }
            }
        }

        return tryEveryExtname(ret);
    }

    function tryLoadNodeModules(context, dirname) {
        dirname = Path.resolve(context, 'node_modules', dirname);
        var packageJsonPath = Path.resolve(dirname, 'package.json');
        try {
            var main = 'index.js';
            // 如果找到了package.json,说明是一个npm的包
            if (existsSync(packageJsonPath)) {
                var packageJson = JSON.parse(fs.readFileSync(packageJsonPath, 'utf8'));
                // 获取包的入口,默认是index.js
                main = packageJson.main || 'index.js';
                var ret = tryEveryExtname(Path.resolve(dirname, main));
                if (ret && existsSync(ret)) {
                    return ret;
                }
            } else { // 反之,是npm包内部的一个文件
                ret = tryEveryExtname(Path.resolve(dirname));
                if (ret && existsSync(ret)) {
                    return ret;
                }
            }
            return null;
        } catch (e) {
            return null;
        }
    }

    function fixDash(path) {
        var alternative = path.replace(rpossibleScssFileName, '_$1');
        if (existsSync(path)) {
            return path;
        } else if (existsSync(alternative)) {
            return alternative;
        }
        return null;
    }

    function tryEveryExtname(path) {
        var extnames = [
            '.js',
            '.scss',
            '.css',
            '/index.js',
            '/index.scss',
            '/index.css',
            'index.js',
            'index.scss',
            'index.css'
        ];
        var existedExt = Path.extname(path);
        if (existedExt) {
            return fixDash(path);
        }

        for (var i = 0; i < extnames.length; i++) {
            var extToBeAppend = extnames[i];
            var renderedPath = normalize(path + extToBeAppend);
            var fixed = fixDash(renderedPath);
            if (fixed) {
                return fixed;
            }
        }

        return null;
    }

    function isScssFile(absPath) {
        // 居然有一种奇葩的写法是require('./'),这里会抛异常
        try {
            return Path.extname(absPath) === '.scss' || Path.extname(absPath) === '.css';
        } catch (e) {
            return false;
        }
    }

    function group(arr) {
        var ret = [];
        for (var i = 0; i < arr.length; i++) {
            if (ret.indexOf(arr[i]) === -1) {
                ret.push(arr[i]);
            }
        }
        return ret;
    }

    function extractAllScssDependencies(entrance, resolve, context, ignore) {
        return group(getScssDependencies(entrance, resolve, context, ignore));
    }

    getScssDependencies = _.memoize(getScssDependencies);

    var ret = '';
    var deps = extractAllScssDependencies(entrance, resolve, context, ignore);
    if (Path.extname(entrance).match(/scss|css/) && deps.indexOf(entrance) === -1) {
        deps.push(entrance);
    }
    deps.forEach(function (dep) {
        var content = fs.readFileSync(dep, 'utf8');
        ret += content
                .replace(/@import\s+(?:url)?(?:(['"])[^'"]+\1;?|\([^)]+\));?/g, '\n')
                .replace(/require\s*\([^)]+\);?/g, '\n')
                .replace(/^\s*\/\/.*(?:[\n\r]|$)/g, '\n')
                .replace(rmultilineComment, '\n')
                .replace(/^\s*@charset\s+(['"])[^'"]+\1;?/g, '\n') + '\n';
    });
    var isCss = deps.reduce(function (ret, dep) {
        if (dep.match(/\.scss$/)) {
            ret = false;
        }
        return ret;
    }, true);

    return { code: ret, deps: deps, isCss: isCss };
}

module.exports = combine;
