var child_process = require('child_process')
var shell = require('shelljs')
var path = require('path')
var fs = require('fs')

// shrinkwrap
var sw = {
    logWarn: function (msg) {
        console.warn('\x1b[33m' + msg + '\x1b[0m')
        console.warn('\x1b[33m' + '[Ykit] 文档请参考：http://ued.qunar.com/ykit/docs-npm%20shrinkwrap.html\n' + '\x1b[0m')
    },
    fileExists: function (filePath) {
        try {
            return fs.statSync(filePath).isFile()
        } catch (err) {
            return false
        }
    },
    createShrinkwrap: function (config) {
        const child = shell.exec('npm shrinkwrap', {
            silent: config.silent,
            async: false
        });

        if (child.code !== 0) {
            if(config.silent) {
                sw.logWarn('\n[Ykit] shrinkwrap 可能遇到问题，建议重新执行 ykit shrinkwrap 并检查是否有错误');
            } else {
                sw.logWarn('\n[Ykit] shrinkwrap 生成遇到错误，详见以上 npm 日志.');
            }
        }
    },
    handlePeerDepRecursive: function (dirPath) {
        if (fs.existsSync(dirPath)) {
            fs.readdirSync(dirPath).forEach(function(file) {
                var currentPath = dirPath + '/' + file
                if (fs.lstatSync(currentPath).isDirectory()) { // recurse
                    sw.handlePeerDepRecursive(currentPath)
                } else {
                    if(path.basename(currentPath) === 'package.json') {
                        var packageJsonObj = require(currentPath)
                        if(packageJsonObj.peerDependencies) {
                            // 保存原来内容
                            fs.writeFileSync(
                                currentPath.replace('package.json', 'ykit.package.json'),
                                JSON.stringify(packageJsonObj, null, '  ')
                            )

                            // 去掉 peerDependencies 保存为 package.json
                            delete packageJsonObj.peerDependencies
                            fs.writeFileSync(
                                currentPath,
                                JSON.stringify(packageJsonObj, null, '  ')
                            )
                        }
                    }
                }
            })
        }
    },
    recoverPeerDepRecursive: function (dirPath) {
        if (fs.existsSync(dirPath)) {
            fs.readdirSync(dirPath).forEach(function(file) {
                var currentPath = dirPath + '/' + file
                if (fs.lstatSync(currentPath).isDirectory()) { // recurse
                    sw.recoverPeerDepRecursive(currentPath)
                } else {
                    if(path.basename(currentPath) === 'ykit.package.json') {
                        var originPkgContent = fs.readFileSync(currentPath)
                        fs.writeFileSync(
                            currentPath.replace('ykit.package.json', 'package.json'),
                            originPkgContent
                        )
                        fs.unlinkSync(currentPath)
                    }
                }
            })
        }
    }
}

module.exports = sw
