'use strict';

var nameReg = /^([^\@]*)(?:\@([^\.]+))?(\.(js|css))$/;
var crypto = require('crypto');
var fs = require('fs');
var path = require('path');
var mkdirp = require('mkdirp');
var cp = require('child_process');

exports.version = require('./package.json').version;

exports.config = function (options, cwd) {
    var self = this;
    // 非常毛的正则匹配
    var rDir = '[^\\.]+\\.([^\\.]+)\\.[^\\.]+$';
    var rPath = new RegExp('^/home/q/jenkins/workspace/[^/]+/' + rDir);
    var matched = cwd.match(rPath);
    var projectDir;

    if (matched && matched[1]) {
        // 匹配到就认为是线上
        projectDir = matched[1];
    } else {
        // 否则认为是本地，取本地路径
        var dir = cwd.split(require('path').sep);
        projectDir = dir[dir.length - 1];
        // 再做一层过滤
        var dirMatched = projectDir.match(new RegExp(rDir));
        if (dirMatched && dirMatched[1]) {
            projectDir = dirMatched[1];
        }
    }

    this.config.output.dev.path = 'dev';
    this.config.output.dev.filename = '[name]@dev[ext]';
    this.config.output.prd.path = 'prd';
    this.config.output.prd.filename = '[name]@[chunkhash][ext]';

    // chunk
    this.config.output.local.chunkFilename = '[id].chunk.js';
    this.config.output.local.publicPath = '//' + path.join('q.qunarzz.com', projectDir, 'prd/');
    this.config.output.dev.chunkFilename = '[id].chunk@dev.js';
    this.config.output.dev.publicPath = '//' + path.join('q.qunarzz.com', projectDir, 'dev/');
    this.config.output.prd.chunkFilename = '[id].chunk@[chunkhash].js';
    this.config.output.prd.publicPath = '//' + path.join('q.qunarzz.com', projectDir, 'prd/');

    this.config.requireRules.push('fekit_modules|fekit.config:main|./src/index.js');

    function md5(content) {
        return crypto.createHash('md5').update(content).digest('hex');
    }

    this.packCallbacks.push(function (opt, stats) {
        if (opt.m || opt.min) {
            var statsInfo = stats.toJson({
                    errorDetails: false
                }),
                verPath = path.join(cwd, 'ver'),
                prdPath = path.join(cwd, 'prd'),
                infos = [],
                chunks = [];

            if (fs.existsSync(verPath)) {
                cp.execSync('rm -rf ' + verPath);
            }
            mkdirp.sync(verPath);

            var assetsInfo = statsInfo.assets.map(function(asset) {
                var name = asset.name,
                    nextName = null;

                if (/\.js$/.test(name) || /\.css$/.test(name)) {
                    var matchInfo = name.match(nameReg),
                        fp = matchInfo[1] + matchInfo[3],
                        version = matchInfo[2],
                        verFp = path.join(verPath, fp + '.ver'),
                        dir = path.dirname(verFp);

                    mkdirp.sync(dir);
                    fs.writeFileSync(verFp, version, 'UTF-8');
                    infos.push(fp + '#' + version);
                    chunks.push('//' + path.join('q.qunarzz.com', projectDir, 'prd/', name));
                }

                asset.name = nextName ? nextName : asset.name;
                return asset;
            }).filter(function(assetInfo) {
                return path.extname(assetInfo.name) === '.js' || path.extname(assetInfo.name) === '.css';
            });

            // 以新的版本号作为资源名称
            self.config.assetsInfo = assetsInfo;
            var chunkName = 'chunk.json';
            var chunkContent = JSON.stringify(chunks);
            var chunkVersion = md5(chunkContent);

            infos.push(chunkName + '#' + chunkVersion);
            fs.writeFileSync(path.join(verPath, chunkName + '.ver'), chunkVersion, 'UTF-8');

            fs.writeFileSync(path.join(verPath, 'versions.mapping'), infos.join('\n'), 'UTF-8');
            fs.writeFileSync(path.join(prdPath, 'chunk@' + chunkVersion + '.json'), chunkContent);
        }
    });

    this.commands.push({
        name: 'source',
        module: require('./commands/source.js')
    }, {
        name: 'sync',
        module: require('./commands/sync.js')
    }, {
        name: 'setup',
        module: require('./commands/setup.js')
    });
};
