/**
 * 这个Plugin的作用是使用md5值来取代原来自增的整数ModuleId
 * 在构建lib.js时，如果采用自增id的形式，当出现lib.js的模块数量变化，但是chunk的内容不变从而chunk的md5也不变的情况时，就会有chunk引用的模块id错乱的情况
 */
require('babel-polyfill');
var createHash = require("crypto").createHash;

function HashedModuleIdsPlugin(options) {
    this.options = Object.assign({
        hashFunction: "md5",
        hashDigest: "base64",
        hashDigestLength: 4
    }, options);
}

HashedModuleIdsPlugin.prototype.apply = function (compiler) {
    var options = this.options;
    compiler.plugin("compilation", function (compilation) {
        var usedIds = new Set();
        compilation.plugin("before-module-ids", function (modules) {
            modules.forEach(function (module) {
                if (module.id === null && module.libIdent && this.options) {
                    var id = module.libIdent({
                        context: this.options.context || compiler.options.context
                    });
                    var hash = createHash(options.hashFunction);
                    hash.update(id);
                    var hashId = hash.digest(options.hashDigest);
                    var len = options.hashDigestLength;
                    while (usedIds.has(hashId.substr(0, len)))
                        len++;
                    module.id = hashId.substr(0, len);
                    usedIds.add(module.id);
                }
            });
        });
    });
}

module.exports = HashedModuleIdsPlugin;
