/**
 * dll打包配置
 * @author jiao.shen
 */
var path = require('path');
var babelQuery = require('../consts').babelQuery;
var HashedModuleIdsPlugin = require('./HashedModuleIdsPlugin');

module.exports = function (self, cwd, isPrd, getVendors) {
    var vendors = [
        'react',
        'react-dom',
        'yo-router',
        'babel-polyfill'
    ];
    vendors = typeof getVendors === 'function' ? getVendors(vendors) : vendors;

    var envPlugin = new self.webpack.DefinePlugin({
        "process.env": {
            NODE_ENV: JSON.stringify(isPrd ? 'production' : 'dev')
        }
    });
    var output = path.resolve(cwd, '.lib_cache');

    var defaultConfig = {
        context: self.config.context,
        output: {
            path: output,
            filename: '[name].js',
            library: '[name]'
        },
        entry: {
            "lib": vendors
        },
        plugins: [
            new self.webpack.DllPlugin({
                path: path.resolve(output, 'manifest.json'),
                name: '[name]',
                context: cwd
            }),
            new HashedModuleIdsPlugin(),
            envPlugin
        ],
        module: {
            loaders: [
                {
                    loader: require.resolve('babel-loader'),
                    test: /\.js$/,
                    exclude: /node_modules/,
                    query: babelQuery
                }
            ]
        }
    }
    // @modify before pass config to webpack
    var libConfigModifier = self.config && self.config.libConfigModifier
    if (typeof libConfigModifier !== 'function') libConfigModifier = function (config) {
        return config
    }
    return libConfigModifier(defaultConfig) || defaultConfig;
};
