var normalize = require('normalize-path');
var path = require('path');
var fs = require('fs');
var sass = require('node-sass-china');
var uglifyJS = require('uglify-js');
var CleanCss = require('clean-css');
var rmdirRecursive = require('rmdir-recursive');

var md5 = require('./util').md5;
var isLibJs = require('./util').isLibJs;
var isExportCss = require('./util').isExportCss;
var execSync = require('child_process').execSync;
var scssExtract = require('../loader/scss-extract');

module.exports = function (cwd, self, dllPlugin) {
    var packService = {
        libPack: function (opt) {
            var min = opt.min || opt.m;
            var destFolder = min ? 'prd' : 'dev';
            var dllDestPath = path.resolve(cwd, destFolder);
            var prdContent = fs.readdirSync(dllDestPath);
            var dllPacked = false;

            for (var i = 0; i < prdContent.length; i++) {
                var filename = prdContent[i];
                if (filename.match(/lib(@.*)?\.js/)) {
                    dllPacked = true;
                    break;
                }
            }

            if (!dllPacked) {
                var minifiedCode = min ? uglifyJS.minify(path.resolve(cwd, '.lib_cache', 'lib.js')).code
                    : fs.readFileSync(path.resolve(cwd, '.lib_cache', 'lib.js'), 'utf8');
                var hash = md5(minifiedCode);
                var libFileName = 'lib@' + (min ? hash : 'dev') + '.js';
                fs.writeFileSync(path.resolve(cwd, destFolder, libFileName), minifiedCode);

                if (min) {
                    var verDest = path.resolve(cwd, 'ver');
                    fs.writeFileSync(path.resolve(verDest, 'lib.js.ver'), hash);
                    fs.appendFileSync(path.resolve(verDest, 'versions.mapping'), '\nlib.js#' + hash);
                }
                console.log('[ykit-config-yo] - lib.min.js packed ->', path.resolve(dllDestPath, libFileName));
                packService.getDll(null, false, self.config.getVendor);
            }
        },
        cssPack: function (option, stats) {
            var statsInfo = stats.toJson({ errorDetails: false });
            statsInfo.assets.map(function (asset) {
                if (asset.name.match(/\.js$/)) {
                    var fakeUrl = '/fake/prd/' + asset.name.replace(/(@.*|\.min)?\.js$/, '');
                    try {
                        var cssCompileRets = packService.compileScssByEntries(fakeUrl, cwd, self);
                        var ret = cssCompileRets[0];
                        if (ret) {
                            var isMinify = option.m || option.min;
                            var code = ret.code;
                            if (isMinify) {
                                code = new CleanCss({
                                    keepSpecialComments: 0
                                }).minify(code).styles;
                            }

                            var hash = md5(code);
                            var version = isMinify ? '@' + hash : '@dev';
                            var destPath = normalize(path.resolve(cwd, ret.key.replace(/^\/prd/, './' + (isMinify ? 'prd' : 'dev')) + version + '.css'));
                            var verPath = path.resolve(cwd, 'ver');
                            var verDest = normalize(path.resolve(cwd, ret.key.replace(/^\/prd/, './ver') + '.css.ver'));
                            var verMapDest = path.resolve(verPath, 'versions.mapping');

                            fs.writeFileSync(destPath, code);

                            if (isMinify) {
                                fs.writeFileSync(verDest, hash);
                                fs.appendFileSync(verMapDest, '\n' + ret.key.replace(/^\/prd\//, '') + '.css#' + hash);
                            }
                            console.log('[ykit-config-yo] - css packed -> ', destPath);
                        }
                    } catch (e) {
                        console.log('[ykit-config-yo]:', 'compile ' + asset.name.replace(/@.*\.js$/, '.css') + ' failed, skip...');
                        console.log(e.stack);
                        throw e;
                        process.exit(1);
                    }
                }
            });
        },
        beforePack: function (done) {
            // 由于采用npm cache安装的模块不会执行postinstall过程，因此这里必须手动执行一次yo3的postinstall。
            var yo3FolderPath = path.resolve(cwd, 'node_modules', 'yo3');
            if (fs.existsSync(yo3FolderPath)) {
                console.log('[ykit-config-yo] - refreshing yo3 style references...');
                execSync('node ' + path.resolve(yo3FolderPath, 'builder', 'postinstall'), { cwd: yo3FolderPath });
            }
            console.log('[ykit-config-yo] - making dll lib.js...');
            rmdirRecursive.sync(path.resolve(cwd, '.lib_cache'));
            fs.mkdirSync(path.resolve(cwd, '.lib_cache'));
            packService.getDll(done, true, self.config.getVendor, cwd, self, dllPlugin);
        },
        compileScssByEntries: function (url) {
            var entries = self.config.entry;
            var compilerContext = self.config.context;
            var ret = [];

            Object.keys(entries).forEach(function (entry) {
                var absPath = normalize(path.resolve(compilerContext, entry));
                var cacheKey = absPath.replace(cwd, '').replace(/\/src/, '/prd').replace(/\.js$/, '');

                if (!url || url.replace(/(@.*)?\.css$/, '').replace(/^\/[^\/]+/, '') === cacheKey) {
                    var extractResult = scssExtract(absPath, self.config.resolve, cwd, self.config.rscsspackage);
                    if (extractResult.deps.length) {
                        var sassRet = sass.renderSync({ data: extractResult.code });
                        ret.push({
                            code: sassRet.css.toString(),
                            key: cacheKey
                        });
                    } else {
                        console.log('[ykit-config-yo]: entry ' + entry + '没有对应的css文件。');
                    }
                }
            });

            return ret;
        },
        getDll: function (done, isMinify, getVendor) {
            if (fs.existsSync(path.resolve(cwd, '.lib_cache'))) {
                rmdirRecursive.sync(path.resolve(cwd, '.lib_cache'));
            }
            var ykitCacheFolder = path.resolve(cwd, '.lib_cache');
            if (!fs.existsSync(ykitCacheFolder)) {
                fs.mkdirSync(ykitCacheFolder);
            }
            // @modify pass self.config  to dll.config
            var dllConfig = require('./dll.config')(self, cwd, isMinify, getVendor);
            self.webpack(dllConfig, function (err) {
                if (!err) {
                    console.log('[ykit-config-yo] - lib.js packed.');
                    // 移除之前添加的dllPlugin然后重新读取manifest, 因为环境可能已经从dev变成了production（执行ykit pack的时候）
                    // 不然会编译出错误的业务代码包
                    if (dllPlugin) {
                        self.config.plugins.splice(self.config.plugins.indexOf(dllPlugin), 1);
                    }
                    var uncachedDllPlugin = new self.webpack.DllReferencePlugin({
                        context: cwd,
                        manifest: JSON.parse(fs.readFileSync(path.resolve(cwd, '.lib_cache', 'manifest.json'), 'utf8'))
                    });
                    self.config.plugins.push(uncachedDllPlugin);
                    done && done();
                }
            });
        },
        devMiddleware: function (req, res, next) {
            try {
                if (isExportCss(req.url)) {
                    res.setHeader('Content-Type', 'text/css; charset=UTF-8');
                    var cssRets = packService.compileScssByEntries(req.url, cwd, self);
                    if (cssRets.length) {
                        res.end(cssRets[0].code);
                    } else {
                        console.error('ykit-config-yo: no css file matched with ' + req.url);
                        next();
                    }
                } else if (isLibJs(req.url)) {
                    var libJsPath = path.resolve(cwd, '.lib_cache', 'lib.js');
                    if (fs.existsSync(libJsPath)) {
                        res.end(fs.readFileSync(libJsPath, 'utf8'));
                    } else {
                        throw new Error('NO_LIB');
                    }
                } else {
                    next();
                }
            } catch (e) {
                if (e.message === 'NO_LIB') {
                    console.log('ykit-config-yo: 没有找到lib.js，将会自动开始构建。');
                    packService.getDll(function () {
                        console.log('ykit-config-yo: lib.js构建成功，请刷新浏览器。');
                    }, false, self.config.getVendor);
                } else {
                    console.error(e.stack);
                }
                next();
            }
        }
    };

    return packService;
}
